<?php
if (!defined('MEDIAWIKI')) die("MediaWiki extensions cannot be run directly.");
if (!defined('WikiDB')) {
	die("The WikiDB extension has not been set up correctly.<br>'"
		. basename(__FILE__) . "' should not be included directly.<br>Instead, "
		. "add the following to LocalSettings.php: "
		. "<tt><b>include_once(\"WikiDB/WikiDB.php\");</b></tt>");
}
/**
 * Special pages provided by WikiDB.
 * The code layout for each IncludableSpecialPage object was originally copied from
 * the NewPages extension by Rob Church, with appropriate modifications.
 *
 * @package MediaWiki
 * @subpackage Extensions
 *
 * @author Mark Clements <mclements at kennel17 dot co dot uk>
 * @copyright Copyright  2006-2013, Mark Clements
 * @license http://creativecommons.org/licenses/by-sa/2.5/ cc-by-sa 2.5 or later
 * @version $Rev: 792 $
 */

WikiDB_RegisterRevision('$Rev: 792 $');

//////////////////////////
// Credit information

	$pWikiDB_SpecialPages = array(
		'Undefined Tables' 	=> "lists tables that have data but no definition",
		'Empty Tables' 		=> "lists tables that do not contain any data",
		'Defined Tables' 	=> "lists all defined tables",
	);

	foreach ($pWikiDB_SpecialPages as $PageName => $Description) {
		$wgExtensionCredits['specialpage'][] = array(
			'name' => $PageName,
			'author' => $pWikiDB_Author,
			'description' => "part of WikiDB - " . $Description,
			'url' => $pWikiDB_URL,
		);
	}


//////////////////////////
// WikiDB_SpecialPage
// This class extends the standard IncludableSpecialPage to add a couple of standard
// functions that WikiDB needs.

class WikiDB_SpecialPage extends IncludableSpecialPage {

	var $pItemsPerPage = 50;

	function pRegister($SpecialPageName) {
	// Old PHP4-style way of calling the constructor - works in MW <= 1.16
		if (is_callable(array("SpecialPage", "SpecialPage"))) {
			SpecialPage::SpecialPage($SpecialPageName, '', true, false, 
									 'default', true);
		}
	// New PHP5-style - required for MW >= 1.17
		else {
			parent::__construct($SpecialPageName, '', true, false, 
								'default', true);
		}
	}

	function pSetupPagination($ItemsPerPage) {
	// Setup items per page
		if (!$ItemsPerPage) {
			global $wgRequest;
			$ItemsPerPage = $wgRequest->getIntOrNull('limit');
			if (!$ItemsPerPage)
				$ItemsPerPage = 50;
		}
		else
			$ItemsPerPage = intval($ItemsPerPage);

		$this->pItemsPerPage = $ItemsPerPage;

	// Setup offset of current page (zero-based).
		$this->pOffset = $wgRequest->getInt('offset');
	}

	function pOutputPagination($TotalRows) {
		global $wgOut, $wgContLang;

		$wgOut->addHTML('<p>' 
						. wfShowingResults($this->pOffset, $this->pItemsPerPage) 
						. '</p>');

		$LastRowOnPage = $this->pOffset + $this->pItemsPerPage;
		if ($LastRowOnPage >= $TotalRows)
			$IsLastPage = true;
		else
			$IsLastPage = false;

		$Output = wfViewPrevNext(
								 $this->pOffset,
								 $this->pItemsPerPage, 
								 $wgContLang->specialPage($this->mName),
								 "", 
								 $IsLastPage
								);

		$wgOut->addHTML( '<p>' . $Output . '</p>' );
	}

	function pGetCount($PartialSQL, $DB) {
		$DBResult = $DB->query("SELECT COUNT(*) AS RowCount FROM " . $PartialSQL);

		$Count = 0;
		if ($DBResult) {
			$Count = $DB->numRows($DBResult);
		}
		
		return $Count;
	}

} // END class WikiDB_SpecialPage


//////////////////////////
// Undefined Tables

class WikiDB_SP_UndefinedTables extends WikiDB_SpecialPage {

	function WikiDB_SP_UndefinedTables() {
		parent::pRegister('UndefinedTables');
	}

	function execute($ItemsPerPage) {
		global $wgOut;

		$this->pSetupPagination($ItemsPerPage);

		$DB =& wfGetDB(DB_SLAVE);
		$tblTables = $DB->tableName(pWIKIDB_tblTables);
		$tblRows = $DB->tableName(pWIKIDB_tblRows);
		$Join =  $tblRows . " LEFT JOIN " .  $tblTables
			  . " ON " . $tblRows . ".table_namespace = "
				. $tblTables . ".table_namespace"
			  . " AND " . $tblRows . ".table_title = " . $tblTables . ".table_title";
		$GroupFields = $tblRows . ".table_namespace, " . $tblRows . ".table_title";
		$Fields = "COUNT(" . $tblRows . ".table_title) AS RecordCount, "
				. $GroupFields;
		$Where = $tblTables . ".table_title IS NULL";

		$PartialSQL = $Join . " WHERE " . $Where . " GROUP BY " . $GroupFields;

		# Don't show the navigation if we're including the page
		if (!$this->mIncluding) {
			$this->setHeaders();
			$wgOut->addWikiText(wfMsg('undefinedtables-header'));

			$RowCount = $this->pGetCount($PartialSQL, $DB);
			$this->pOutputPagination($RowCount);
		}

		$res = $DB->query("SELECT " . $Fields . " FROM " . $PartialSQL 
						  . " ORDER BY RecordCount DESC, " 
						  . $tblRows . ".table_namespace, "
						  . $tblRows . ".table_title "
						  . " LIMIT " . $this->pOffset . "," . $this->pItemsPerPage);
			 
		$RowCount = $DB->numRows($res);
		if ($RowCount > 0) {
		// Make list
			$wgOut->addHTML("<ol>\n");
			while($row = $DB->fetchObject($res))
				$wgOut->addHTML($this->pMakeListItem($row));
			$wgOut->addHTML("</ol>\n");
		}
		else
			$wgOut->addWikiText(wfMsg('undefinedtables-none'));
		$DB->freeResult($res);
	}

	function pMakeListItem($Row) {
		global $wgUser;

		$Title = Title::makeTitleSafe($Row->table_namespace, $Row->table_title);
		if (!is_null($Title)) {
			$skin = $wgUser->getSkin();
			$DefLink = $skin->makeLinkObj($Title);
			$DataLink = $Title->getLocalURL('action=viewdata');
			$DataLinkText = wfMsg('undefinedtables-record', $Row->RecordCount);

			return("<li>" . $DefLink . ' - <a href="' . $DataLink . '">'
				   . $DataLinkText . '</a></li>' . "\n");
		}
		else {
			return("<!-- Invalid title " . htmlspecialchars($Row->table_title)
				 . " in namespace " . htmlspecialchars($Row->table_namespace)
				 . " -->\n");
		}
	}

} // END class WikiDB_SP_UndefinedTables


//////////////////////////
// Empty Tables

class WikiDB_SP_EmptyTables extends WikiDB_SpecialPage {

	function WikiDB_SP_EmptyTables() {
		parent::pRegister('EmptyTables');
	}

	function execute($ItemsPerPage) {
		global $wgOut;

		$this->pSetupPagination($ItemsPerPage);

		$DB =& wfGetDB(DB_SLAVE);
		$tblTables = $DB->tableName(pWIKIDB_tblTables);
		$tblRows = $DB->tableName(pWIKIDB_tblRows);
		$Join =  $tblTables . " LEFT JOIN " .  $tblRows
			  . " ON " . $tblTables . ".table_namespace = "
				. $tblRows . ".table_namespace"
			  . " AND " . $tblTables . ".table_title = " . $tblRows . ".table_title";
		$GroupFields = $tblTables . ".table_namespace, "
					 . $tblTables . ".table_title";
		$Fields = $GroupFields;
		$Where = $tblRows . ".table_title IS NULL";

		$PartialSQL = $Join . " WHERE " . $Where . " GROUP BY " . $GroupFields;

		# Don't show the navigation if we're including the page
		if (!$this->mIncluding) {
			$this->setHeaders();
			$wgOut->addWikiText(wfMsg('emptytables-header'));

			$RowCount = $this->pGetCount($PartialSQL, $DB);
			$this->pOutputPagination($RowCount);
		}

		$res = $DB->query("SELECT " . $Fields . " FROM " . $PartialSQL
						  . " ORDER BY " . $tblTables . ".table_namespace, "
						  . $tblTables . ".table_title"
						  . " LIMIT " . $this->pOffset . "," . $this->pItemsPerPage);

		$RowCount = $DB->numRows($res);
		if ($RowCount > 0) {
		// Make list
			$wgOut->addHTML("<ol>\n");
			while($row = $DB->fetchObject($res))
				$wgOut->addHTML($this->pMakeListItem($row));
			$wgOut->addHTML("</ol>\n");
		}
		else
			$wgOut->addWikiText(wfMsg('emptytables-none'));
		$DB->freeResult($res);
	}

	function pMakeListItem($Row) {
		global $wgUser;

		$Title = Title::makeTitleSafe($Row->table_namespace, $Row->table_title);
		if (!is_null($Title)) {
			$skin = $wgUser->getSkin();
			$DefLink = $skin->makeLinkObj($Title);
			return( "<li>" . $DefLink . '</li>' . "\n");
		}
		else {
			return("<!-- Invalid title " . htmlspecialchars($Row->table_title)
				 . " in namespace " . htmlspecialchars($Row->table_namespace)
				 . " -->\n");
		}
	}

} // END class WikiDB_SP_EmptyTables


//////////////////////////
// All Tables

class WikiDB_SP_AllTables extends WikiDB_SpecialPage {

	function WikiDB_SP_AllTables() {
		parent::pRegister('AllTables');
	}

	function execute($ItemsPerPage) {
		global $wgOut;

		$this->pSetupPagination($ItemsPerPage);

		$DB =& wfGetDB(DB_SLAVE);
		$tblTables = $DB->tableName(pWIKIDB_tblTables);
		$tblRows = $DB->tableName(pWIKIDB_tblRows);
		$Join =  $tblTables . " LEFT JOIN " .  $tblRows
			  . " ON " . $tblTables . ".table_namespace = "
					. $tblRows . ".table_namespace"
			  . " AND " . $tblTables . ".table_title = " . $tblRows . ".table_title";
		$GroupFields = $tblTables . ".table_namespace, "
					 . $tblTables . ".table_title";
		$Fields = "COUNT(" . $tblRows . ".table_title) AS RecordCount, "
				. $GroupFields;

		$PartialSQL = $Join . " GROUP BY " . $GroupFields;
					
		# Don't show the navigation if we're including the page
		if (!$this->mIncluding) {
			$this->setHeaders();
			$wgOut->addWikiText(wfMsg('alltables-header'));

			$RowCount = $this->pGetCount($PartialSQL, $DB);
			$this->pOutputPagination($RowCount);
		}

		$res = $DB->query("SELECT " . $Fields . " FROM " . $PartialSQL 
						  . " ORDER BY " . $tblTables . ".table_namespace, "
						  . $tblTables . ".table_title"
						  . " LIMIT " . $this->pOffset . "," . $this->pItemsPerPage);

		$RowCount = $DB->numRows($res);
		if ($RowCount > 0) {
		// Make list
			$wgOut->addHTML("<ol>\n");
			while($row = $DB->fetchObject($res))
				$wgOut->addHTML($this->pMakeListItem($row));
			$wgOut->addHTML("</ol>\n");
		}
		else
			$wgOut->addWikiText(wfMsg('alltables-none'));
		$DB->freeResult($res);
	}

	function pMakeListItem($Row) {
		global $wgUser;

		$Title = Title::makeTitleSafe($Row->table_namespace, $Row->table_title);
		if (!is_null($Title)) {
			$skin = $wgUser->getSkin();
			$DefLink = $skin->makeLinkObj($Title);
			$DataLink = $Title->getLocalURL('action=viewdata');
			$DataLinkText = wfMsg('alltables-record', $Row->RecordCount);

			if ($Row->RecordCount == 0)
				$Class = ' class="new"';
			else
				$Class = '';

			return("<li>" . $DefLink . ' - <a href="' . $DataLink . '"'
				   . $Class . '>' . $DataLinkText . '</a></li>' . "\n");
		}
		else {
			return("<!-- Invalid title " . htmlspecialchars($Row->table_title)
				 . " in namespace " . htmlspecialchars($Row->table_namespace)
				 . " -->\n");
		}
	}

} // END class WikiDB_SP_AllTables
